#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Set lib version from `pyproject.toml`.

.. note::

  .. deprecated:: 1.3.6
    Use:

    .. code-block:: python
  
      __version__ = __import__("importlib.metadata", fromlist=["version"]).version(
          <package name>
      )
  
    instead 

This library allows for setting the version number for a library from
the pyproject.toml file.

Add to your `pyproject.toml` add a new section:

.. code-block:: toml

  [tool.berhoel.helper.set_version]
  version_files = ["berhoel/helper/_version.py"]

Generate the version file:

.. code-block:: shell

  > poetry run set_lib_version
  writing berhoel/helper/_version.py

In the library `__init__.py` just use:

.. code-block:: python

  try:
      from ._version import __version__
  except ImportError:
      __version__ = "0.0.0.invalid0"
"""

# Standard library imports.
import argparse
import warnings
from pathlib import Path

# Third party library imports.
import tomli

# Local library imports.
from . import __version__

__date__ = "2023/12/26 19:03:33 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020, 2022 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


def build_parser():
    "Build cli parser."
    parser = argparse.ArgumentParser(
        prog="set_lib_version",
        description="Create version files with version number from `pyproject.toml`.",
    )
    parser.add_argument(
        "--version", action="version", version=f"%(prog)s {__version__}"
    )
    return parser


def build():
    """Create version files with version number from `pyproject.toml`."""

    warnings.warn(message, DeprecationWarning)

    parser = build_parser()
    parser.parse_args()

    PYPROJECT = (Path() / "pyproject.toml").resolve()

    with PYPROJECT.open("rb") as conf_inp:
        CONFIG = tomli.load(conf_inp)

    VERSION = CONFIG["tool"]["poetry"]["version"]
    VER_FILES = CONFIG["tool"]["berhoel"]["helper"]["set_version"]["version_files"]
    for ver_file in VER_FILES:
        print(f"writing {ver_file!s}")
        with Path(ver_file).resolve().open("w") as target:
            target.write(f'__version__ = "{VERSION}"\n')


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "50/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Proccess arguments flexible.

This module provides a class named ``CheckArgs`` and a function named
``check_args``.

The class ``CheckArgs`` is ment to be used as base class for own
classes.  After the ``__init__`` method is called a dictionary args is
added to the local namespace.

The init mathod has four arguments.

The first is, as usual, the ``self`` handler.  The second is a list or
tuples holding the allowed keyword arguments as first tuple element
and their default values as remaining tuple elements.

The third and forth argument are the ``*data`` and ``**kw`` arguments
from the implemented class.

Example:

>>> from .check_args import CheckArgs
>>> class samp(CheckArgs):
...     default = (('A', 'A'), ('B', 'B'), ('C', ('C', 1, 2)))
...     def __init__(self, *data, **kw):
...         CheckArgs.__init__(self, *data, **kw)
...         print(self.args['A'], self.args['B'], self.args['C'])
...
>>> A = samp(A = 34)
34 B ('C', 1, 2)
>>> A = samp(34, "HALLO")
34 HALLO ('C', 1, 2)
>>>

The function ``check_args`` provides similar funtionality for
functions.  It also takes four arguments.

The first to third argument are simlar to the second to forth element
of the class.  The forth element of the function is the functions
name.

Example:

>>> from .check_args import check_args
>>> def tst(*data, **kw):
...     default = (('A', 'A'), ('B', 'B'), ('C', ('C', 1, 2)))
...     return check_args(default, data, kw, 'tst')
...
>>> tst(A = 34) == {'A': 34, 'B': 'B', 'C': ('C', 1, 2)}
True
>>> tst(34) == {'A': 34, 'B': 'B', 'C': ('C', 1, 2)}
True
>>>
"""

__date__ = "2023/03/25 15:30:09 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 1998-2000,2018 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


class ArgError(TypeError):
    """Error from the argument checking."""

    pass


class CheckArgs:
    """A wrapper around the ``check_args`` function.

    Args:
        default (tuple[tuple[str, object]]): argument names and default values for ``__init__`` method.
        data (tuple): arguments to ``__init__`` method.
        kw (dict): keyword arguments to ``__init__`` method.

    Returns:
        dict: argument name, value pairs."""

    def __init__(self, *data, **kw):
        self.args = check_args(self.default, data, kw, self.__class__.__name__)


def check_args(default, data, kw, name):
    """Check arguments for function.

    Args:
        default (tuple[tuple[str, object]]): argument names and default values for function call.
        data (tuple): arguments to function call.
        kw (dict): keyword arguments to function call.
        name (str): name for reporting.

    Returns:
        dict: argument name, value pairs."""
    args = dict(default)
    arglen = len(default)
    if (len(data) + len(kw)) > arglen:
        raise ArgError(
            "%s requires at most %d argument; %d given" % (name, arglen, len(data))
        )
    arglist = []
    for i in range(len(data)):
        args[default[i][0]] = data[i]
        arglist.append(default[i][0])
    for k in arglist:
        if k in kw:
            raise ArgError(
                "%s: key '%s' used in **kw argument is also set by *data." % (name, k)
            )
    args.update(kw)
    if len(args) != arglen:
        out = list(args.keys())
        for key, dummy in default:
            out.remove(key)
        raise ArgError("%s: unexpected keyword argument: %s" % (name, out[0]))
    return args


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "80/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Misc helper
"""

# Standard library imports.
import time
from enum import Enum, auto
from datetime import timedelta
from contextlib import contextmanager

__date__ = "2023/12/26 19:13:35 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 1999, 2000, 2021, 2022 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "bhoel@starship.python.net"
__version__ = __import__("importlib.metadata", fromlist=["version"]).version(
    "bhoelHelper"
)


class SwirlSelect(Enum):
    LINES = auto()
    DOTS = auto()


__swirl_string = {
    SwirlSelect.LINES: r"\|/-",
    SwirlSelect.DOTS: (
        "\N{BRAILLE PATTERN DOTS-123}"  # "⠇"
        "\N{BRAILLE PATTERN DOTS-237}"  # "⡆"
        "\N{BRAILLE PATTERN DOTS-378}"  # "⣄"
        "\N{BRAILLE PATTERN DOTS-678}"  # "⣠"
        "\N{BRAILLE PATTERN DOTS-568}"  # "⢰"
        "\N{BRAILLE PATTERN DOTS-456}"  # "⠸"
        "\N{BRAILLE PATTERN DOTS-145}"  # "⠙"
        "\N{BRAILLE PATTERN DOTS-124}"  # "⠋"
    ),
}


def swirl(style=SwirlSelect.LINES):
    """Generator to show a swirling life indicator.

    >>> sw = swirl()
    >>> a = [_ for _ in zip(range(5), sw)]
    \\...|.../...-...\\...
    >>> sw = swirl(SwirlSelect.DOTS)
    >>> a = [_ for _ in zip(range(8), sw)]
    ⠇...⡆...⣄...⣠...⢰...⠸...⠙...⠋...

    Returns:
      `generator`: printing running indicator.
    """
    sw_string = __swirl_string[style]
    while True:
        for c in sw_string:
            print(c, end="\r")
            yield


def count_with_msg(msg="loop", start=0):
    """Counting variable with start value and message.

    >>> c = count_with_msg("msg", 5)
    >>> print([i for _, i in zip(range(5),c)] == [5, 6, 7, 8, 9])
    msg 1 ...msg 2 ...msg 3 ...msg 4 ...msg 5 ...True
    >>>

    Args:
        msg (str): base message
        start (int): counter start_time

    Returns:
        `generator`: counter with message.
    """
    i = 1
    _count = start
    while True:
        print("{} {} ".format(msg, i), end="\r")
        yield _count
        _count += 1
        i += 1


@contextmanager
def process_msg_context(msg):
    """Provides a context for calling routines and reporting entering and exit.

    >>> with process_msg_context("do something"):
    ...     pass
    do something......do something...done
    >>>

    Args:
        msg (str): message for bracing process.

    Returns:
        `contextmanager`: bracing message.
    """
    print("{}...".format(msg), end="\r")
    yield
    print("{}...done".format(msg))


@contextmanager
def timed_process_msg_context(msg, time_form=None):
    """Provides a context for calling routines and reporting entering and exit.
    Report spent time.

    >>> with timed_process_msg_context("do something"):
    ...     time.sleep(1)
    do something......do something...done (0:00:01)
    >>> with timed_process_msg_context("do something", lambda t:"{:d}s".format(int(t))):
    ...     time.sleep(1)
    do something......do something...done (1s)
    >>>

    Args:
        msg (str): message for bracing process.
        time_form (func): function formatting druntime.

    Returns:
        `contextmanager`: bracing message.
    """
    if time_form is None:
        time_form = lambda t: timedelta(seconds=int(t))
    start_time = time.time()
    print("{}...".format(msg), end="\r")
    yield
    print(("{}...done ({})").format(msg, time_form(time.time() - start_time)))


# Local Variables:
# mode: python
# compile-command: "python ../../setup.py test"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Determine machine-specific parameters affecting floating-point
arithmetic.

Function to determine machine-specific parameters affecting
floating-point arithmetic.

* This is build after "NUMERICAL RECIPES in C", second edition,
  Reprinted 1996, pp.  889.
"""

# Standard library imports.
import math
from functools import lru_cache

__date__ = "2022/07/30 19:09:33 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 1999, 2000,2018 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "bhoel@starship.python.net"


def machar():
    """
    Determines and returns machine-specific parameters affecting
    floating-point arithmetic.

    Returns:
      dict: Values in result dictionary are:

      ``ibeta`` (int)
        The radix in which numbers are represented, almost
        always 2, but occasionally 16, or even 10
      ``it`` (int)
        The number of base-``ibeta`` digits in the floating point
        mantissa
      ``machep`` (int)
        Is the exponent of the smallest (most negative) power if ``ibeta``
        that, added to 1.0 gives something different from 1.0.
      ``eps`` (float)
        Is the floating-point number ``ibeta`` ** ``machdep``, loosly
        referred to as the *floating-point precision*.
      ``negep`` (int)
        Is the exponenet of the smalles power of ``ibeta`` that, subtracted from
        1.0, gives something different from 1.0.
      ``epsneg`` (float)
        Is ``ibeta`` ** ``negep``, another way of defining floating-point precision.
        Not infrequently ``epsneg`` is 0.5 times ``eps``; occasionally ``eps`` and ``epsneg``
        are equal.
      ``iexp`` (int)
        Is the number of bits in the exponent (including its sign or bias)
      ``minexp`` (int)
        Is the smallest (most negative) power if ``ibeta`` consistent with there no
        leading zeros in the mantissa.
      ``xmin`` (float)
        Is the floating-point number ``ibeta`` ** ``minexp``, generally the smallest
        (in magnitude) usable floating value
      ``maxexp`` (int)
        Is the smallest (positive) power of ``ibeta`` that causes overflow.
      ``xmax`` (float)
        Is (1 - ``epsneg``) x ``ibeta`` ** ``maxexp``, generally the largest (in magnitude)
        usable floating value.
      ``irnd`` (int)
        Returns a code in the range 0...5, giving information on what kind of rounding is done
        in addition, and on how underflow is handled.

        If ``irnd`` returns 2 or 5, then your computer is compilant with the IEEE standard for
        rounding. If it returns 1 or 4, then it is doing some kind of rounding, but not the IEEE
        standard. If ``irnd`` returns 0 or 3, then it is truncating the result, not rounding it.
      ``ngrd`` (int)
        Is the number of *guard digits* used when truncating the product of two mantissas to fit
        the representation

    This is taken from "NUMERICAL RECIPES in C", second edition,
    Reprinted 1996.
    """
    one = float(1)
    two = one + one
    zero = one - one
    # Determine ``ibeta`` and ``beta`` by the method of M. Malcom.
    a = temp1 = one
    while temp1 - one == zero:
        a += a
        temp = a + one
        temp1 = temp - a
    b = one
    itemp = 0
    while itemp == 0:
        b += b
        temp = a + b
        itemp = int(temp - a)
    ibeta = itemp
    beta = float(ibeta)
    # Determine ``it`` and ``irnd``.
    it = 0
    b = temp1 = one
    while temp1 - one == zero:
        it = it + 1
        b = beta * b
        temp = b + one
        temp1 = temp - b
    irnd = 0
    betah = beta / two
    temp = a + betah
    if temp - a != zero:
        irnd = 1
    tempa = a + beta
    temp = tempa + betah
    if irnd == 0 and temp - tempa != zero:
        irnd = 2
    # Determine ``negep`` und ``epsneg``.
    negep = it + 3
    betain = one / beta
    a = one
    i = 1
    while i <= negep:
        i = i + 1
        a = betain * a
    b = a
    while 1:
        temp = one - a
        if temp - one != zero:
            break
        a = beta * a
        negep = negep - 1
    negep = -negep
    epsneg = a
    # Determine ``machdep`` and ``eps``.
    machdep = -it - 3
    a = b
    while 1:
        temp = one + a
        if temp - one != zero:
            break
        a = a * beta
        machdep = machdep + 1
    eps = a
    # Deterrmine ``ngrd``.
    ngrd = 0
    temp = one + eps
    if irnd == 0 and temp * one - one != zero:
        ngrd = 1
    # Determine ``iexp``.
    i = 0
    k = 1
    z = betain
    t = one + eps
    nxres = 0
    while 1:  # Loop until underflow ocurs, then exit.
        y = z
        z = y * y
        a = z * one  # check for underflow
        temp = z * t
        if a + a == zero or math.fabs(z) >= y:
            break
        temp1 = temp * betain
        if temp1 * beta == z:
            break
        i = i + 1
        k = k + k
    if ibeta != 10:
        iexp = i + 1
        mx = k + k
    else:  # For decimal machines only
        iexp = i + i
        iz = ibeta
        while k >= iz:
            iz = ibeta * iz
            iexp = iexp + 1
        mx = iz + iz - 1
    # To determine ``minexp`` and ``xmin``, loop until an underflow
    # occurs, then exit.
    while 1:
        xmin = y
        y = betain * y
        a = y * one  # Check here for the underflow
        temp = y * t
        if a + a != zero or math.fabs(y) < xmin:
            k = k + 1
            temp1 = temp * betain
            if temp1 * beta == y and temp != y:
                nxres = 3
                xmin = y
                break
        else:
            break
    minexp = -k
    # Determine maxexp, xmax.
    if mx <= k + k + 3 and ibeta != 10:
        mx = mx + mx
        iexp = iexp + 1
    maxexp = mx + minexp
    irnd = nxres + irnd  # Adjust ``irnd`` to reflect partial underflow
    if irnd >= 2:
        maxexp = maxexp - 2  # Adjust for IEEE-stype machines
    i = maxexp + minexp
    # Adjust for machines with implicit leading bit in binary mantissa,
    # and machines with radix point at extreme right of mantissa.
    if ibeta == 2 and not i:
        maxexp = maxexp - 1
    if i > 20:
        maxexp = maxexp - 1
    if a != y:
        maxexp = maxexp - 2
    xmax = one - epsneg
    if xmax * one != xmax:
        xmax = one - beta * epsneg
    xmax = xmax / (xmin * beta * beta * beta)
    i = maxexp + minexp + 3
    j = 1
    while j <= i:
        j = j + 1
        if ibeta == 2:
            xmax = xmax + xmax
        else:
            xmax = xmax * beta
    return {
        "ibeta": ibeta,
        "it": it,
        "irnd": irnd,
        "ngrd": ngrd,
        "machdep": machdep,
        "negep": negep,
        "iexp": iexp,
        "minexp": minexp,
        "maxexp": maxexp,
        "eps": eps,
        "epsneg": epsneg,
        "xmin": xmin,
        "xmax": xmax,
    }


__tmp = machar()

ibeta = __tmp["ibeta"]
"""``ibeta`` (int)
  The radix in which numbers are represented, almost
  always 2, but occasionally 16, or even 10
"""

it = __tmp["it"]
"""``it`` (int)
  The number of base-``ibeta`` digits in the floating point
  mantissa
"""

machdep = __tmp["machdep"]
"""``machep`` (int)
  Is the exponent of the smallest (most negative) power if ``ibeta``
  that, added to 1.0 gives something different from 1.0.
"""

eps = __tmp["eps"]
"""``eps`` (float)
  Is the floating-point number ``ibeta`` ** ``machdep``, loosly
  referred to as the *floating-point precision*.
"""

negep = __tmp["negep"]
"""``negep`` (int)
  Is the exponenet of the smalles power of ``ibeta`` that, subtracted from
  1.0, gives something different from 1.0.
"""

epsneg = __tmp["epsneg"]
"""``epsneg`` (float)
  Is ``ibeta`` ** ``negep``, another way of defining floating-point precision.
  Not infrequently ``epsneg`` is 0.5 times ``eps``; occasionally ``eps`` and ``epsneg``
  are equal.
"""

iexp = __tmp["iexp"]
"""``iexp`` (int)
  Is the number of bits in the exponent (including its sign or bias)
"""

minexp = __tmp["minexp"]
"""``minexp`` (int)
  Is the smallest (most negative) power if ``ibeta`` consistent with there no
  leading zeros in the mantissa.
"""

xmin = __tmp["xmin"]
"""``xmin`` (float)
  Is the floating-point number ``ibeta`` ** ``minexp``, generally the smallest
  (in magnitude) usable floating value
"""

maxexp = __tmp["maxexp"]
"""``maxexp`` (int)
  Is the smallest (positive) power of ``ibeta`` that causes overflow.
"""

xmax = __tmp["xmax"]
"""``xmax`` (float)
  Is (1 - ``epsneg``) x ``ibeta`` ** ``maxexp``, generally the largest (in magnitude)
  usable floating value.
"""

irnd = __tmp["irnd"]
"""``irnd`` (int)
  Returns a code in the range 0...5, giving information on what kind of rounding is done
  in addition, and on how underflow is handled.

  If ``irnd`` returns 2 or 5, then your computer is compilant with the IEEE standard for
  rounding. If it returns 1 or 4, then it is doing some kind of rounding, but not the IEEE
  standard. If ``irnd`` returns 0 or 3, then it is truncating the result, not rounding it.
"""

ngrd = __tmp["ngrd"]
"""``ngrd`` (int)
  Is the number of *guard digits* used when truncating the product of two mantissas to fit
  the representation"
"""


# Local Variables:
# mode: python
# compile-command: "python ../../setup.py test"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Initialization for berhoel python namespace packages.
"""

__date__ = "2023/06/10 21:19:49 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2018 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"

__path__ = __import__("pkgutil").extend_path(__path__, __name__)


# Local Variables:
# mode: python
# compile-command: "python ../setup.py test"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `berhoel.helper`.
"""

# Standard library imports.
import time
from pathlib import Path

# Third party library imports.
import toml
import pytest

# First party library imports.
from berhoel import helper

__date__ = "2022/07/19 18:28:02 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


@pytest.fixture
def base_path():
    "Return path to project base."
    return Path(__file__).parents[3]


@pytest.fixture
def py_project(base_path):
    "Return path of project `pyproject.toml`."
    return base_path / "pyproject.toml"


@pytest.fixture
def toml_inst(py_project):
    "Return `toml` instance of project `pyproject.toml`"
    return toml.load(py_project.open("r"))


def test_version(toml_inst):
    "Test for consistent version numbers."
    assert helper.__version__ == toml_inst["tool"]["poetry"]["version"]


def test_swirl(capsys):
    "Test swirler."
    swirl = helper.swirl()
    for expected in r"\|/-\|":
        next(swirl)
        res = capsys.readouterr()
        assert res.out == "{}\r".format(expected)


def test_swirl_dots(capsys):
    "Test swirler with (braille) dots."
    swirl = helper.swirl(helper.SwirlSelect.DOTS)
    for expected in "⠇⡆⣄⣠⢰⠸⠙⠋":
        next(swirl)
        res = capsys.readouterr()
        assert res.out == "{}\r".format(expected)


def test_count_with_message_1(capsys):
    "Test counter with life message."
    count = helper.count_with_msg()
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i == j
        assert res.out == "loop {} \r".format(i + 1)
        if i > 5:
            break


def test_count_with_message_2(capsys):
    "Test counter with life message."
    count = helper.count_with_msg(start=10)
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i + 10 == j
        assert res.out == "loop {} \r".format(i + 1)
        if i > 5:
            break


def test_count_with_message_3(capsys):
    "Test counter with life message."
    count = helper.count_with_msg(msg="msg")
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i == j
        assert res.out == "msg {} \r".format(i + 1)
        if i > 5:
            break


def test_count_with_message_4(capsys):
    "Test counter with life message."
    count = helper.count_with_msg("alt", 5)
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i + 5 == j
        assert res.out == "alt {} \r".format(i + 1)
        if i > 5:
            break


def test_process_msg_context(capsys):
    "Test process context."
    with helper.process_msg_context("do something"):
        res = capsys.readouterr()
        assert res.out == "do something...\r"
    res = capsys.readouterr()
    assert res.out == "do something...done\n"


def test_timed_process_msg_context_1(capsys):
    "Test timed process context."
    with helper.timed_process_msg_context("do something"):
        res = capsys.readouterr()
        time.sleep(1)
        assert res.out == "do something...\r"
    res = capsys.readouterr()
    assert res.out == "do something...done (0:00:01)\n"


def test_timed_process_msg_context_2(capsys):
    "Test timed process context."
    with helper.timed_process_msg_context(
        "do something", lambda t: "{:d}s".format(int(t))
    ):
        res = capsys.readouterr()
        time.sleep(1)
        assert res.out == "do something...\r"
    res = capsys.readouterr()
    assert res.out == "do something...done (1s)\n"


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Testing `berhoel.helper.machar`
"""

# First party library imports.
from berhoel.helper import machar

__date__ = "2021/12/11 17:50:51 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


def test_ibeta():
    assert isinstance(machar.ibeta, int)


def test_it():
    assert isinstance(machar.it, int)


def test_irnd():
    assert isinstance(machar.irnd, int)


def test_ngrd():
    assert isinstance(machar.ngrd, int)


def test_machdep():
    assert isinstance(machar.machdep, int)


def test_negep():
    assert isinstance(machar.negep, int)


def test_iexp():
    assert isinstance(machar.iexp, int)


def test_minexp():
    assert isinstance(machar.minexp, int)


def test_maxexp():
    assert isinstance(machar.maxexp, int)


def test_eps():
    assert isinstance(machar.eps, float)


def test_epsneg():
    assert isinstance(machar.epsneg, float)


def test_xmin():
    assert isinstance(machar.xmin, float)


def test_xmax():
    assert isinstance(machar.xmax, float)


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for `berhoel.helper.check_args`.
"""

# Third party library imports.
import pytest

# First party library imports.
from berhoel.helper.check_args import ArgError, CheckArgs, check_args

__date__ = "2023/03/25 15:33:33 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


class Samp(CheckArgs):
    default = (("A", "A"), ("B", "B"), ("C", ("C", 1, 2)))

    def __init__(self, *data, **kw):
        CheckArgs.__init__(self, *data, **kw)

    def __call__(self):
        return (self.args["A"], self.args["B"], self.args["C"])


def test_samp_1():
    probe = Samp(A=34)
    assert probe() == (34, "B", ("C", 1, 2))


def test_samp_2():
    probe = Samp(34, "HALLO")
    assert probe() == (34, ("HALLO"), ("C", 1, 2))


def test_samp_3():
    with pytest.raises(ArgError):
        Samp(D=34)


def tst(*data, **kw):
    default = (("A", "A"), ("B", "B"), ("C", ("C", 1, 2)))
    return check_args(default, data, kw, "tst")


def test_tst_0():
    assert tst() == {"A": "A", "B": "B", "C": ("C", 1, 2)}


def test_tst_1():
    assert tst(A=34) == {"A": 34, "B": "B", "C": ("C", 1, 2)}


def test_tst_2():
    tst(34, B=22) == {"A": 34, "B": 22, "C": ("C", 1, 2)}


def test_tst_3():
    tst(34) == {"A": 34, "B": "B", "C": ("C", 1, 2)}


def test_tst_4():
    with pytest.raises(ArgError):
        tst(D=34)


def test_tst_5():
    with pytest.raises(ArgError):
        tst(34, 35, B=36)


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:
